@@ -59,7 +59,7 @@ int main(int argc, char **argv)
 		exit(-1);
 	}
 
-	putenv("CONFIG_KEYDATA=/etc/");
+	putenv((char *)"CONFIG_KEYDATA=/etc/");
 	if(argc == 2)
 		etc.load(argv[1]);
 	else
