@@ -1,5 +1,6 @@
 // Copyright (C) 1999-2005 Open Source Telecom Corporation.
-// Copyright (C) 2006-2008 David Sugar, Tycho Softworks.
+// Copyright (C) 2006-2009 David Sugar, Tycho Softworks,
+// Copyright (C) 2009 Leandro Melo de Sales <leandroal@gmail.com>
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -75,6 +76,20 @@ typedef int SOCKET;
 #define	MSG_NOSIGNAL	0
 #endif
 
+#ifndef SOCK_DCCP
+#define SOCK_DCCP       6
+#endif
+#ifndef IPPROTO_DCCP
+#define IPPROTO_DCCP    33
+#endif
+#ifndef SOL_DCCP
+#define SOL_DCCP       269
+#endif
+#define DCCP_SOCKOPT_AVAILABLE_CCIDS    12
+#define DCCP_SOCKOPT_CCID               13
+#define DCCP_SOCKOPT_TX_CCID            14
+#define DCCP_SOCKOPT_RX_CCID            15
+
 #ifdef	CCXX_NAMESPACES
 namespace ost {
 #endif
@@ -676,6 +691,143 @@ public:
 	virtual bool isPending(Pending pend, timeout_t timeout = TIMEOUT_INF);
 };
 
+/**
+ * DCCP sockets are used for stream based connected sessions between two
+ * sockets.  Both error recovery and flow control operate transparently
+ * for a DCCP socket connection.  The DCCP socket base class is used both
+ * for client connections and to bind a DCCP "server" for accepting DCCP 
+ * streams.
+ *
+ * An implicit and unique DCCPSocket object exists in Common C++ to represent
+ * a bound DCCP socket acting as a "server" for receiving connection requests.
+ * This class is not part of DCCPStream because such objects normally perform
+ * no physical I/O (read or write operations) other than to specify a listen
+ * backlog queue and perform "accept" operations for pending connections.
+ * The Common C++ DCCPSocket offers a Peek method to examine where the next
+ * pending connection is coming from, and a Reject method to flush the next
+ * request from the queue without having to create a session.
+ *
+ * The DCCPSocket also supports a "OnAccept" method which can be called when a
+ * DCCPStream related object is created from a DCCPSocket.  By creating a
+ * DCCPStream from a DCCPSocket, an accept operation automatically occurs, and
+ * the DCCPSocket can then still reject the client connection through the
+ * return status of it's OnAccept method.
+ *
+ * @author Leandro Sales <leandroal@gmail.com>
+ * @author Heverton Stuart <hevertonsns@gmail.com>
+ * @short bound server for DCCP streams and sessions.
+ */
+class __EXPORT DCCPSocket : public Socket
+{
+	union {
+		struct sockaddr_in ipv4;
+#ifdef	CCXX_IPV6
+		struct sockaddr_in6 ipv6;
+#endif
+	}	peer;
+
+	Family family;
+	
+public:
+	/**
+	 * A method to call in a derived DCCPSocket class that is acting
+	 * as a server when a connection request is being accepted.  The
+	 * server can implement protocol specific rules to exclude the
+	 * remote socket from being accepted by returning false.  The
+	 * Peek method can also be used for this purpose.
+	 *
+	 * @return true if client should be accepted.
+	 * @param ia internet host address of the client.
+	 * @param port number of the client.
+	 */
+	virtual bool onAccept(const IPV4Host &ia, tpport_t port);
+#ifdef	CCXX_IPV6
+	virtual bool onAccept(const IPV6Host &ia, tpport_t port);
+#endif
+
+	/**
+	 * A DCCP "server" is created as a DCCP socket that is bound
+	 * to a hardware address and port number on the local machine
+	 * and that has a backlog queue to listen for remote connection
+	 * requests.  If the server cannot be created, an exception is
+	 * thrown.
+	 *
+	 * @param bind local ip address or interface to use.
+	 * @param port number to bind socket under.
+	 * @param backlog size of connection request queue.
+	 */
+	DCCPSocket(const IPV4Address &bind, tpport_t port, unsigned backlog = 5);
+#ifdef	CCXX_IPV6
+	DCCPSocket(const IPV6Address &bind, tpport_t port, unsigned backlog = 5);
+#endif
+
+	/**
+	 * Create a named dccp socket by service and/or interface id.
+	 * For IPV4 we use [host:]svc or [host/]svc for the string.
+	 * If we have getaddrinfo, we use that to obtain the addr to
+	 * bind for.
+	 *
+	 * @param name of host interface and service port to bind.
+	 * @param backlog size of connection request queue.
+	 */
+	DCCPSocket(const char *name, Family family = IPV4, unsigned backlog = 5);
+
+	/**
+	 * Create an unconnected ephemeral DCCP client socket.
+	 */
+	DCCPSocket(Family family = IPV4);
+
+	/**
+	 * Create a server session by accepting a DCCP Socket.
+	 */
+	DCCPSocket(DCCPSocket& server, timeout_t timeout = 0);
+
+	/**
+	 * Used to reject the next incoming connection request.
+     */
+	void reject(void);
+
+	/**
+	 * Disconnect active dccp connection (client use).
+	 */
+	void disconnect(void);
+
+	/**
+	 * Set CCID DCCP.
+	 */
+	bool setCCID(uint8_t ccid);
+
+	/**
+	 * Create a DCCP client connection to a DCCP socket (on
+	 * a remote machine).
+	 *
+	 * @param host address of remote DCCP server.
+	 * @param port number to connect.
+	 */
+	void connect(const IPV4Host &host, tpport_t port, timeout_t timeout = 0);
+#ifdef	CCXX_IPV6
+	void connect(const IPV6Host &host, tpport_t port, timeout_t timeout = 0);
+#endif
+
+	/**
+	 * Connect to a named client.
+	 */
+	void connect(const char *name);
+
+	/**
+	 * Used to wait for pending connection requests.
+	 * @return true if data packets available.
+	 * @param timeout in milliseconds. TIMEOUT_INF if not specified.
+	 */
+	inline bool isPendingConnection(timeout_t timeout = TIMEOUT_INF) /* not const -- jfc */
+		{return Socket::isPending(Socket::pendingInput, timeout);}
+
+	/**
+	 * Use base socket handler for ending this socket.
+	 */
+	virtual ~DCCPSocket();
+};
+
 /**
  * UDP sockets implement the TCP SOCK_DGRAM UDP protocol.  They can be
  * used to pass unverified messages between hosts, or to broadcast a
