@@ -1,4 +1,4 @@
-// Copyright (C) 2001-2005 Gianni Mariani
+// Copyright (C) 2001-2008 Gianni Mariani
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -289,7 +289,7 @@ void CommandOptionNoArg::foundOption( CommandOptionParse * cop, const char * val
 class CommandOptionParse_impl : public CommandOptionParse {
 public:
 
-	char			* comment;
+	const char	* comment;
 	int				  num_options;
 	struct option   * long_options;
 	CommandOption  ** opt_list;
@@ -317,7 +317,7 @@ public:
 	CommandOptionParse_impl(
 		int				in_argc,
 		char		 ** in_argv,
-		char		  * in_comment,
+		const char		  * in_comment,
 		CommandOption * options
 	) :
 		comment( in_comment ),
@@ -629,7 +629,7 @@ public:
 CommandOptionParse * makeCommandOptionParse(
 	int				   argc,
 	char			** argv,
-	char			 * comment,
+	const char		 * comment,
 	CommandOption	 * options
 ) {
 	return new CommandOptionParse_impl( argc, argv, comment, options );
