@@ -1,4 +1,4 @@
-// Copyright (C) 2004 TintaDigital - STI, LDA.
+// Copyright (C) 2004-2008 TintaDigital - STI, LDA.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -86,7 +86,7 @@ natResult natv6Lookup( SOCKET sfd, struct sockaddr_in6 * nat );
  * @param res result from NATLookup function.
  * @return error description.
  */
-char * natErrorString( natResult res );
+const char * natErrorString( natResult res );
 
 #ifdef	CCXX_NAMESPACES
 }
