@@ -1,4 +1,5 @@
 // Copyright (C) 1999-2005 Open Source Telecom Corporation.
+// Copyright (C) 2006-2008 David Sugar, Tycho Softworks.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -69,10 +70,18 @@ Semaphore::Semaphore(unsigned resource)
 
 Semaphore::~Semaphore()
 {
+	// Unlock is needed to unlock the mutex in the case of a cancel during Semaphore::wait()
+	// see PTHREAD_COND_TIMEDWAIT(3P) 
+	pthread_mutex_unlock(&_mutex);
 	pthread_cond_destroy(&_cond);
 	pthread_mutex_destroy(&_mutex);
 }
 
+void Semaphore::force_unlock_after_cancellation()
+{
+   pthread_mutex_unlock(&_mutex);
+}
+
 bool Semaphore::wait(timeout_t timeout)
 {
 	struct timespec ts;
