@@ -1,5 +1,6 @@
 // Copyright (C) 1999-2005 Open Source Telecom Corporation.
-// Copyright (C) 2006-2008 David Sugar, Tycho Softworks.
+// Copyright (C) 2006-2009 David Sugar, Tycho Softworks,
+// Copyright (C) 2009 Leandro Melo de Sales <leandroal@gmail.com>
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -2255,6 +2256,526 @@ void TCPV6Socket::reject(void)
 }
 #endif
 
+DCCPSocket::DCCPSocket(Family fam) :
+Socket(fam, SOCK_DCCP, IPPROTO_DCCP)
+{
+	family = fam;
+}
+
+#ifdef	HAVE_GETADDRINFO
+DCCPSocket::DCCPSocket(const char *name, Family fam, unsigned backlog) :
+Socket(fam, SOCK_DCCP, IPPROTO_DCCP)
+{
+	char namebuf[128], *cp;
+	struct addrinfo hint, *list = NULL, *first;
+	snprintf(namebuf, sizeof(namebuf), "%s", name);
+	cp = strrchr(namebuf, '/');
+	if(!cp)
+		cp = strrchr(namebuf, ':');
+
+	if(!cp) {
+		cp = namebuf;
+		name = NULL;
+	}
+	else {
+		name = namebuf;
+		*(cp++) = 0;
+		if(!strcmp(name, "*"))
+			name = NULL;
+	}
+
+	family = fam;
+	memset(&hint, 0, sizeof(hint));
+	hint.ai_family = family;
+	hint.ai_socktype = SOCK_DCCP;
+	hint.ai_protocol = IPPROTO_DCCP;
+	hint.ai_flags = AI_PASSIVE;
+
+	if(getaddrinfo(name, cp, &hint, &list) || !list) {
+		endSocket();
+		error(errBindingFailed, (char *)"Could not find service", errno);
+		return;
+	}
+
+#if defined(SO_REUSEADDR)
+	int opt = 1;
+	setsockopt(so, SOL_SOCKET, SO_REUSEADDR, (char *)&opt,
+			(socklen_t)sizeof(opt));
+#endif
+
+	first = list;
+	while(list) {
+		if(!bind(so, list->ai_addr, (socklen_t)list->ai_addrlen)) {
+			state = BOUND;
+			break;
+		}
+		list = list->ai_next;
+	}
+	freeaddrinfo(first);
+	if(state != BOUND) {
+		endSocket();
+		error(errBindingFailed,(char *)"Could not bind socket",socket_errno);
+		return;
+	}
+
+	if(listen(so, backlog)) {
+		endSocket();
+		error(errBindingFailed,(char *)"Could not listen on socket",socket_errno);
+		return;
+	}
+}
+#else
+DCCPSocket::DCCPSocket(const char *name, Family fam, unsigned backlog) :
+Socket(fam, SOCK_DCCP, IPPROTO_DCCP)
+{
+	char namebuf[128], *cp;
+	struct sockaddr_in addr;
+#ifdef	CCXX_IPV6
+	struct sockaddr_in6 addr6;
+#endif
+	struct sockaddr *ap;
+	socklen_t = alen = sizeof(addr);
+
+	struct servent *svc;
+
+	family = fam;
+	memset(&addr, 0, sizeof(addr));
+	snprintf(namebuf, sizeof(namebuf), "%s", name);
+	cp = strrchr(namebuf, '/');
+	if(!cp)
+		cp = strrchr(namebuf, ':');
+
+	if(!cp) {
+		cp = namebuf;
+		name = "*";
+	}
+	else {
+		name = namebuf;
+		*(cp++) = 0;
+	}
+
+	addr.sin_family = family;
+	if(isdigit(*cp))
+		addr.sin_port = htons(atoi(cp));
+	else {
+		mutex.enter();
+		svc = getservbyname(cp, "dccp");
+		if(svc)
+			addr.sin_port = svc->s_port;
+		mutex.leave();
+		if(!svc) {
+			endSocket();
+			error(errBindingFailed, "Could not find service", errno);
+			return;
+
+		}
+	}
+
+#if defined(SO_REUSEADDR)
+	int opt = 1;
+	setsockopt(so, SOL_SOCKET, SO_REUSEADDR, (char *)&opt,
+			(socklen_t)sizeof(opt));
+#endif
+
+	switch(family) {
+#ifdef	CCXX_IPV6
+	case IPV6:
+		IPV6Address ia6(name);
+		addr6.sin6_port = addr.sin_port;
+		addr6.sin6_family = family;
+		ap = &addr6;
+		alen = sizeof(addr6);
+		break;
+#endif
+	default:
+		IPV4Address ia(name);
+		addr.sin_addr = getaddress(ia);
+		ap = &addr;
+	}
+
+	if(bind(so, ap, alen)) {
+		endSocket();
+		error(errBindingFailed,(char *)"Could not bind socket",socket_errno);
+		return;
+	}
+
+	if(listen(so, backlog)) {
+		endSocket();
+		error(errBindingFailed,(char *)"Could not listen on socket",
+				socket_errno);
+		return;
+	}
+	state = BOUND;
+}
+#endif
+
+DCCPSocket::DCCPSocket(const IPV4Address &ia, tpport_t port, unsigned backlog) :
+Socket(AF_INET, SOCK_DCCP, IPPROTO_DCCP)
+{
+	struct sockaddr_in addr;
+
+	memset(&addr, 0, sizeof(addr));
+	addr.sin_family = AF_INET;
+	addr.sin_addr = getaddress(ia);
+	addr.sin_port = htons(port);
+	family = IPV4;
+
+	memset(&peer, 0, sizeof(peer));
+	peer.ipv4 = addr;
+
+#if defined(SO_REUSEADDR)
+	int opt = 1;
+	setsockopt(so, SOL_SOCKET, SO_REUSEADDR, (char *)&opt, (socklen_t)sizeof(opt));
+#endif
+	if(bind(so, (struct sockaddr *)&addr, sizeof(addr))) {
+		endSocket();
+		error(errBindingFailed,(char *)"Could not bind socket",socket_errno);
+		return;
+	}
+
+	if(listen(so, backlog)) {
+		endSocket();
+		error(errBindingFailed,(char *)"Could not listen on socket",socket_errno);
+		return;
+	}
+	state = BOUND;
+}
+
+#ifdef CCXX_IPV6
+DCCPSocket::DCCPSocket(const IPV6Address &ia, tpport_t port, unsigned backlog) :
+Socket(AF_INET6, SOCK_DCCP, IPPROTO_DCCP)
+{
+	struct sockaddr_in6 addr;
+
+	memset(&addr, 0, sizeof(addr));
+	addr.sin6_family = AF_INET6;
+	addr.sin6_addr = getaddress(ia);
+	addr.sin6_port = htons(port);
+
+	family = IPV6;
+	memset(&peer, 0, sizeof(peer));
+	peer.ipv6 = addr;
+
+#if defined(SO_REUSEADDR)
+	int opt = 1;
+	setsockopt(so, SOL_SOCKET, SO_REUSEADDR, (char *)&opt, (socklen_t)sizeof(opt));
+#endif
+	if(bind(so, (struct sockaddr *)&addr, sizeof(addr))) {
+		endSocket();
+		error(errBindingFailed,(char *)"Could not bind socket",socket_errno);
+		return;
+	}
+
+	if(listen(so, backlog)) {
+		endSocket();
+		error(errBindingFailed,(char *)"Could not listen on socket",socket_errno);
+		return;
+	}
+	state = BOUND;
+}
+
+bool DCCPSocket::onAccept(const IPV6Host &ia, tpport_t port)
+{
+	return true;
+}
+
+#endif
+
+bool DCCPSocket::onAccept(const IPV4Host &ia, tpport_t port)
+{
+	return true;
+}
+
+DCCPSocket::~DCCPSocket()
+{
+	endSocket();
+}
+
+void DCCPSocket::disconnect(void)
+{
+	if(Socket::state != CONNECTED)
+		return;
+
+	endSocket();
+	so = socket(family, SOCK_DCCP, IPPROTO_DCCP);
+	if(so != INVALID_SOCKET)
+		Socket::state = AVAILABLE;
+}
+
+#ifdef	HAVE_GETADDRINFO
+void DCCPSocket::connect(const char *target)
+{
+	char namebuf[128];
+	char *cp;
+	struct addrinfo hint, *list = NULL, *next, *first;
+	bool connected = false;
+
+	snprintf(namebuf, sizeof(namebuf), "%s", target);
+	cp = strrchr(namebuf, '/');
+	if(!cp)
+		cp = strrchr(namebuf, ':');
+
+	if(!cp) {
+		connectError();
+		return;
+	}
+
+	*(cp++) = 0;
+
+	memset(&hint, 0, sizeof(hint));
+	hint.ai_family = family;
+	hint.ai_socktype = SOCK_DCCP;
+	hint.ai_protocol = IPPROTO_DCCP;
+
+	if(getaddrinfo(namebuf, cp, &hint, &list) || !list) {
+		connectError();
+		return;
+	}
+
+	first = list;
+
+	while(list) {
+		if(!::connect(so, list->ai_addr, (socklen_t)list->ai_addrlen)) {
+			connected = true;
+			break;
+		}
+		next = list->ai_next;
+		list = next;
+	}
+
+	freeaddrinfo(first);
+
+	if(!connected) {
+		connectError();
+		return;
+	}
+
+	Socket::state = CONNECTED;
+}
+#else
+void DCCPSocket::connect(const char *target)
+{
+	char namebuf[128];
+	char *cp;
+	bool connected = false;
+	struct servent *svc;
+	tpport_t port;
+
+	snprintf(namebuf, sizeof(namebuf), "%s", target);
+	cp = strrchr(namebuf, '/');
+	if(!cp)
+		cp = strrchr(namebuf, ':');
+
+	if(!cp) {
+		connectError();
+		return;
+	}
+
+	*(cp++) = 0;
+
+	if(isdigit(*cp))
+		port = atoi(cp);
+	else {
+		mutex.enter();
+		svc = getservbyname(cp, "dccp");
+		if(svc)
+			port = ntohs(svc->s_port);
+		mutex.leave();
+		if(!svc) {
+			connectError();
+			return;
+		}
+	}
+
+	switch(family) {
+	case IPV4:
+		connect(IPV4Host(namebuf), port);
+		break;
+#ifdef	CCXX_IPV6
+	case IPV6:
+		connect(IPV6Host(namebuf), port);
+		break;
+#endif
+	default:
+		connectError();
+	}
+}
+#endif
+
+void DCCPSocket::connect(const IPV4Host &host, tpport_t port, timeout_t timeout)
+{
+	size_t i;
+	fd_set fds;
+	struct timeval to;
+	bool connected = false;
+	int rtn;
+	long sockopt;
+	socklen_t len = sizeof(sockopt);
+
+	for(i = 0 ; i < host.getAddressCount(); i++) {
+		struct sockaddr_in addr;
+		memset(&addr, 0, sizeof(addr));
+		addr.sin_family = AF_INET;
+		addr.sin_addr = host.getAddress(i);
+		addr.sin_port = htons(port);
+
+		if(timeout)
+			setCompletion(false);
+
+		// Win32 will crash if you try to connect to INADDR_ANY.
+		if ( INADDR_ANY == addr.sin_addr.s_addr )
+			addr.sin_addr.s_addr = INADDR_LOOPBACK;
+		rtn = ::connect(so, (struct sockaddr *)&addr, (socklen_t)sizeof(addr));
+		if(!rtn) {
+			connected = true;
+			break;
+		}
+
+#ifndef WIN32
+		if(errno == EINPROGRESS)
+#else
+			if(WSAGetLastError() == WSAEINPROGRESS)
+#endif
+			{
+				FD_ZERO(&fds);
+				FD_SET(so, &fds);
+				to.tv_sec = timeout / 1000;
+				to.tv_usec = timeout % 1000 * 1000;
+
+				// timeout check for connect completion
+
+				if(::select((int)so + 1, NULL, &fds, NULL, &to) < 1)
+					continue;
+
+				getsockopt(so, SOL_SOCKET, SO_ERROR, (char *)&sockopt, &len);
+				if(!sockopt) {
+					connected = true;
+					break;
+				}
+				endSocket();
+				so = socket(AF_INET, SOCK_DCCP, IPPROTO_DCCP);
+				if(so == INVALID_SOCKET)
+					break;
+			}
+	}
+
+	setCompletion(true);
+	if(!connected) {
+		rtn = errno;
+		errno = rtn;
+		connectError();
+		return;
+	}
+
+	Socket::state = CONNECTED;
+}
+
+#ifdef  CCXX_IPV6
+void DCCPSocket::connect(const IPV6Host &host, tpport_t port, timeout_t timeout)
+{
+	size_t i;
+	fd_set fds;
+	struct timeval to;
+	bool connected = false;
+	int rtn;
+	long sockopt;
+	socklen_t len = sizeof(sockopt);
+
+	for(i = 0 ; i < host.getAddressCount(); i++) {
+		struct sockaddr_in6 addr;
+		memset(&addr, 0, sizeof(addr));
+		addr.sin6_family = AF_INET6;
+		addr.sin6_addr = host.getAddress(i);
+		addr.sin6_port = htons(port);
+
+		if(timeout)
+			setCompletion(false);
+
+		// Win32 will crash if you try to connect to INADDR_ANY.
+		if ( !memcmp(&addr.sin6_addr, &in6addr_any, sizeof(in6addr_any)))
+			memcpy(&addr.sin6_addr, &in6addr_loopback, sizeof(in6addr_loopback));
+		rtn = ::connect(so, (struct sockaddr *)&addr, (socklen_t)sizeof(addr));
+		if(!rtn) {
+			connected = true;
+			break;
+		}
+
+#ifndef WIN32
+		if(errno == EINPROGRESS)
+#else
+			if(WSAGetLastError() == WSAEINPROGRESS)
+#endif
+			{
+				FD_ZERO(&fds);
+				FD_SET(so, &fds);
+				to.tv_sec = timeout / 1000;
+				to.tv_usec = timeout % 1000 * 1000;
+
+				// timeout check for connect completion
+
+				if(::select((int)so + 1, NULL, &fds, NULL, &to) < 1)
+					continue;
+
+				getsockopt(so, SOL_SOCKET, SO_ERROR, (char *)&sockopt, &len);
+				if(!sockopt) {
+					connected = true;
+					break;
+				}
+				endSocket();
+				so = socket(AF_INET6, SOCK_DCCP, IPPROTO_DCCP);
+				if(so == INVALID_SOCKET)
+					break;
+			}
+	}
+
+	setCompletion(true);
+	if(!connected) {
+		rtn = errno;
+		errno = rtn;
+		connectError();
+		return;
+	}
+
+	Socket::state = CONNECTED;
+}
+#endif
+
+bool DCCPSocket::setCCID(uint8_t ccid)
+{
+	uint8_t ccids[4];             /* for getting the available CCIDs, should be large enough */
+	socklen_t len = sizeof (ccids);
+	int ret;
+	bool ccid_supported = false;
+
+	/*
+	 * Determine which CCIDs are available on the host
+	 */
+	ret = getsockopt (so, SOL_DCCP, DCCP_SOCKOPT_AVAILABLE_CCIDS, &ccids, &len);
+	if (ret < 0) {
+		cout << "Can not determine available CCIDs" << endl;
+		return false;
+	}
+
+	for (unsigned i = 0; i < sizeof (ccids); i++) {
+		if (ccid == ccids[i]) {
+			ccid_supported = true;
+			break;
+		}
+	}
+
+	if (!ccid_supported) {
+		cout << "CCID specified is not supported" << endl;
+		return false;
+	}
+
+	if (setsockopt (so, SOL_DCCP, DCCP_SOCKOPT_CCID, &ccid, sizeof (ccid)) < 0) {
+		cout << "Can not set CCID" << endl;
+		return false;
+	}
+
+	return true;
+}
+
 TCPStream::TCPStream(TCPSocket &server, bool throwflag, timeout_t to) :
 	streambuf(), Socket(accept(server.getSocket(), NULL, NULL)),
 #ifdef	HAVE_OLD_IOSTREAM
@@ -3014,7 +3535,7 @@ int TCPStream::overflow(int c)
 		pbump(1);
 	}
 	return c;
-			}
+}
 
 TCPSession::TCPSession(const IPV4Host &ia,
 		       tpport_t port, size_t size, int pri, size_t stack) :
@@ -3057,7 +3578,7 @@ Thread(pri, stack), TCPStream(IPV4)
 
 	setCompletion(true);
 	Socket::state = CONNECTED;
-				}
+}
 
 #ifdef  CCXX_IPV6
 TCPSession::TCPSession(const IPV6Host &ia,
@@ -3102,7 +3623,7 @@ Thread(pri, stack), TCPStream(IPV6)
 
 	setCompletion(true);
 	Socket::state = CONNECTED;
-					}
+}
 #endif
 
 
@@ -3111,7 +3632,7 @@ Thread(pri, stack), TCPStream(s)
 {
 	setCompletion(true);
 	setError(false);
-					}
+}
 
 #ifdef  CCXX_IPV6
 TCPSession::TCPSession(TCPV6Socket &s, int pri, size_t stack) :
@@ -3119,7 +3640,7 @@ Thread(pri, stack), TCPStream(s)
 {
 	setCompletion(true);
 	setError(false);
-					}
+}
 #endif
 
 TCPSession::~TCPSession()
@@ -3155,19 +3676,19 @@ int TCPSession::waitConnection(timeout_t timer)
 	}
 	Socket::state = CONNECTED;
 	return 0;
-					}
+}
 
 void TCPSession::initial(void)
 {
 	if(waitConnection(60000))
 		exit();
-					}
+}
 
 ostream& operator<<(ostream &os, const IPV4Address &ia)
 {
 	os << inet_ntoa(getaddress(ia));
 	return os;
-					}
+}
 
 #ifdef WIN32
 init_WSA::init_WSA()
